/* Copyright (C) Teemu Suutari */

#include <vector>

#include "onekpaq_common.h"

#include "AsmDecode.hpp"
#include "Timer.hpp"

extern "C" void onekpaq_decompressor_mode1(void *src,void *dest);
extern "C" void onekpaq_decompressor_mode2(void *src,void *dest);
extern "C" void onekpaq_decompressor_mode3(void *src,void *dest);
extern "C" void onekpaq_decompressor_mode4(void *src,void *dest);

std::vector<u8> AsmDecode(const std::vector<u8> &src1,const std::vector<u8> &src2,StreamCodec::EncodeMode mode) {
	typedef void (*AsmDecompressor)(void*,void*);
	AsmDecompressor decompressors[]={onekpaq_decompressor_mode1,onekpaq_decompressor_mode2,
		onekpaq_decompressor_mode3,onekpaq_decompressor_mode4};

	// we have no way of calculating the real size for dest without decompressing.
	// lets just put some huge number here
	uint expectedSize=262144;

	std::vector<u8> ret(expectedSize);
	std::vector<u8> combine=src1;
	combine.insert(combine.end(),src2.begin(),src2.end());
	const uint destStartMargin=9;

	INFO("Running asm decompressor Xbx=%p Xdi=%p",combine.data()+src1.size(),ret.data()+destStartMargin);
	auto timeTaken=Timer([&]() {
			decompressors[static_cast<uint>(mode)-1](combine.data()+src1.size(),ret.data()+destStartMargin);
	});
	INFO("Asm decompressor done. Time taken %f seconds",float(timeTaken));

	return std::vector<u8>(ret.begin()+destStartMargin,ret.begin()+expectedSize-destStartMargin);
}
